//------------------------------------------------------
// Copyright 2000-2006, GarageGames.com, Inc.
// Written, modfied, or otherwise interpreted by Ed Maurina, Hall Of Worlds, LLC
//------------------------------------------------------
 //-----------------------------------------------------------------------------

datablock FlyingVehicleData(BoxFlyer)  
{
   category               = "baseVehicles";
   shapeFile              = "~/data/shapes/Vehicles/boxflyer.dts";

   renderWhenDestroyed    = false;

   cameraRoll             = false; // Roll the camera with the vehicle
   cameraMaxDist          = 30;    // Far distance from vehicle
   cameraOffset           = 8;     // Vertical offset from camera mount point
   cameraLag              = 0.1;   // Velocity lag of camera
   cameraDecay            = 0.75;  // Decay per sec. rate of velocity lag

   observeThroughObject   = true; 
   useEyePoint            = true;  // Use the eye point of the player mounted to the vehicle, 
                                   // not the vehicle's eye point
   drag                   = 0.0;
   density                = 0.4;

   MinDrag                = 88;    // Linear Drag (eventually slows you down when not thrusting...constant drag)
   RotationalDrag         = 10;    // Anguler Drag (dampens the drift after you stop moving the mouse...also tumble drag)
   RechargeRate           = 0.0;
   
   minDrag                = 85;    // Linear Drag (eventually slows you down when not thrusting...constant drag)

   rotationalDrag         = 15;    // Anguler Drag (dampens the drift after you stop moving the mouse...also tumble drag)

   rechargeRate           = 50.0;

   maxAutoSpeed           = 15000; // Autostabilizer kicks in when less than this speed. (world units/second)
   autoAngularForce       = 1000;   // Angular stabilizer force (this force levels you out when autostabilizer kicks in)
   autoLinearForce        = 0;     // Linear stabilzer force (this slows you down when autostabilizer kicks in)
   autoInputDamping       = 0.95;  // Dampen control input so you don't` whack out at very slow speeds

   // Maneuvering
   maxSteeringAngle       = 1.2;   // Max radians you can rotate the wheel. Smaller number is more maneuverable.
   horizontalSurfaceForce = 2400;  // Horizontal center "wing" (provides "bite" into the wind for climbing/diving and turning)
   verticalSurfaceForce   = 4000;  // Vertical center "wing" (controls side slip. lower numbers make MORE slide.)
   maneuveringForce       = 24000; // Horizontal jets (W,S,D,A key thrust)

   steeringForce          = 2400;  // Steering jets (force applied when you move the mouse)

   steeringRollForce      = 400;   // Steering jets (how much you heel over when you turn)
 
   rollForce              = 100;   // Auto-roll (self-correction to right you after you roll/invert)
   hoverHeight            = 3.5;   // Height off the ground at rest
   createHoverHeight      = 2.5;   // Height off the ground when created

   gravityMod             = 1;

   integration            = 60;    // Physics integration: TickSec/Rate
   collisionTol           = 0.1;   // Collision distance tolerance
   contactTol             = 0.1;   // Contact velocity tolerance

   mass                   = 500;   // Mass of the vehicle

   bodyFriction           = 0;     // Don't mess with this.
   bodyRestitution        = 0;   // When you hit the ground, how much you rebound. (between 0 and 1)
   minRollSpeed           = 0;     // Don't mess with this.
   minImpactSpeed         = 1;     // If hit ground at speed above this then it's an impact. world units/second
   softImpactSpeed        = 10;    // Sound hooks. This is the soft hit.
   hardImpactSpeed        = 25;    // Sound hooks. This is the hard hit.

   collDamageThresholdVel = 20.0;
   collDamageMultiplier   = 0.02;

   mountable = true;

};

function BoxFlyer::onImpact(%this, %obj, %collidedObject, %vec, %vecLen)
{
   %obj.damage(0, VectorAdd(%obj.getPosition(),%vec),
      %vecLen * %this.speedDamageScale, "Impact");

}

function BoxFlyer::onDamage()
{
    
}

function BoxFlyer::onAdd(%this,%obj)
{
    //parent::onAdd(%this,%obj);
    %obj.mountImage(ShipbowImage,0);
    %obj.setImageAmmo(0,1);
}

datablock ShapeBaseImageData(ShipbowImage)
{
   // Basic Item properties
   shapeFile = "~/data/shapes/projectiles/shipbow.dts";
   emap = true;

   // Specify mount point & offset for 3rd person, and eye offset
   // for first person rendering.
   mountPoint = 0;
   offset = "0 0 0";
   correctMuzzleVector = false;
   className = "WeaponImage";

   projectile = CrossbowProjectile;
   projectileType = Projectile;
	fireTimeout = 300;

   // Initial start up state
   stateName[0]                     = "Preactivate";
   stateTransitionOnLoaded[0]       = "Activate";
   stateTransitionOnNoAmmo[0]       = "Activate";

   // Activating the gun.  Called when the weapon is first
   // mounted and there is ammo.
   stateName[1]                     = "Activate";
   stateTransitionOnTimeout[1]      = "Ready";
   stateTimeoutValue[1]             = 0.6;
   stateSequence[1]                 = "Activate";

   // Ready to fire, just waiting for the trigger
   stateName[2]                     = "Ready";
   stateTransitionOnNoAmmo[2]       = "Activate";
   stateTransitionOnTriggerDown[2]  = "Fire";

   // Fire the weapon. Calls the fire script which does 
   // the actual work.
   stateName[3]                     = "Fire";
   stateTransitionOnTimeout[3]      = "Reload";
   stateTimeoutValue[3]             = 0.2;
   stateFire[3]                     = true;
   stateRecoil[3]                   = LightRecoil;
   stateAllowImageChange[3]         = false;
   stateSequence[3]                 = "Fire";
   stateScript[3]                   = "onFire";

   // Play the relead animation, and transition into
   stateName[4]                     = "Reload";
   stateTransitionOnNoAmmo[4]       = "Activate";
   stateTransitionOnTimeout[4]      = "Ready";
   stateTimeoutValue[4]             = 0.8;
   stateAllowImageChange[4]         = false;
   stateSequence[4]                 = "Reload";
   stateEjectShell[4]               = true;

   // No ammo in the weapon, just idle until something
   // shows up. Play the dry fire sound if the trigger is
   // pulled.
   stateName[5]                     = "Activate";
   stateTransitionOnAmmo[5]         = "Reload";
   stateSequence[5]                 = "Activate";
   stateTransitionOnTriggerDown[5]  = "DryFire";

   // No ammo dry fire
   stateName[6]                     = "DryFire";
   stateTimeoutValue[6]             = 1.0;
   stateTransitionOnTimeout[6]      = "Activate";
};

function BoxFlyer::onTrigger(%data, %obj, %trigger, %state)
{
   // data = datablock
   // obj = object number
   // trigger = 0 for "fire", 1 for "jump", 3 for "thrust"
   // state = 1 for firing, 0 for not firing
   if(%trigger == 0)
   {
      switch (%state) {
         case 0:
            %obj.fireWeapon = false;
            %obj.setImageTrigger(0, false);
            %obj.setImageTrigger(1, false);
         case 1:
            %obj.fireWeapon = true;
            if(%obj.nextWeaponFire == 0) {
               %obj.setImageTrigger(0, true);
               %obj.setImageTrigger(1, false);
            }
            else {
               %obj.setImageTrigger(0, false);
               %obj.setImageTrigger(1, true);
            }
      }
   }
}

function BoxFlyer::onImpact(%this, %obj, %collidedObject, %vec, %vecLen)
{
    /*$Game::Player.damage(0, VectorAdd(%obj.getPosition(),%vec),
      %vecLen * %this.speedDamageScale, "Impact");*/

    getMountNodeObject(0).applyDamage(%obj.directDamage);
}

function BoxFlyer::onFire(%data,%obj,%slot)
{
   Parent::onFire(%data,%obj,%slot);
   %obj.nextWeaponFire = 1;
   schedule(%data.fireTimeout, 0, "fireNextGun", %obj);   
}

function ShipbowImage::onTriggerDown(%this, %obj, %slot)
{
}

function ShipbowImage::onTriggerUp(%this, %obj, %slot)
{
}

function fireNextGun(%obj)
{
   if(%obj.fireWeapon)
   {
      if(%obj.nextWeaponFire == 0)
      {
         %obj.setImageTrigger(0, true);
         %obj.setImageTrigger(1, false);
      }
      else
      {
         %obj.setImageTrigger(0, false);
         %obj.setImageTrigger(1, true);
      }
   }
   else
   {
      %obj.setImageTrigger(0, false);
      %obj.setImageTrigger(1, false);
   }
}

function ShapeBaseImageData::onFire(%data, %obj, %slot)
{
	%projectile = %data.projectile;
	
	%muzzleVector = %obj.getMuzzleVector(%slot);
   %objectVelocity = %obj.getVelocity();
   
   %muzzleVelocity = VectorAdd(
      VectorScale(%muzzleVector, %projectile.muzzleVelocity),%objectVelocity
      /*VectorScale(%objectVelocity, %projectile.velInheritFactor)*/);
   
   if($DIFFICULTY == 2){
        %dDamage        = 50;
        %rDamage        = 20;
   }else if($DIFFICULTY == 1){
        %dDamage        = 20;
        %rDamage        = 10;
   }else{
        %dDamage       = 5;
        %rDamage        = 3;
   }

    echo("Diff: " @ %dDamage);
       %p = new (%data.projectileType)() {
         dataBlock        = %data.projectile;
         initialVelocity  = %muzzleVelocity;
         initialDirection = %obj.getMuzzleVector(%slot);
         initialPosition  = %obj.getMuzzlePoint(%slot);
         sourceObject     = %obj;
         sourceSlot       = %slot;
         vehicleObject    = %vehicle;
         directDamage = %dDamage;
         radiusDamage = %rDamage;

      };

    echo("P: " @ %p.directDamage);
 
   if (isObject(%obj.lastProjectile) && %obj.deleteLastProjectile)
      %obj.lastProjectile.delete();

   %obj.lastProjectile = %p;
   %obj.deleteLastProjectile = %data.deleteLastProjectile;
   
   MissionCleanup.add(%p);
 
   return %p;
}